#pragma once

#include "types.h"
#include "const.h"
#include "sha3.h"
#include "chacha20.h"

typedef struct passman_log_entry_s {
  
  u8 is_used;
  
  u32 id;
  
  ascii data[PASSMAN_MAX_LOG_DATA];
  
  u8 hash[SHA3_256_HASH_LEN];
  
} __attribute__((packed)) passman_log_entry_t;

typedef struct passman_log_s
{ 
  passman_log_entry_t entries[PASSMAN_MAX_LOG_ENTRIES];
  
  u64 iter;
  u8 salt[PASSMAN_SALT_LEN];
  u8 nonce[CHACHA20_NONCE_LEN];
  u8 entries_hmac[SHA3_256_HASH_LEN];
  u8 hmac[SHA3_256_HASH_LEN];
  
} __attribute__((packed)) passman_log_t;

passman_log_t *read_log(ascii *fpath);
u8 write_log(ascii *fpath, passman_log_t *log);
void generate_log_key(u8 *password, u64 password_len, passman_log_t *log, u8 *key);
u8 generate_log_hmac(passman_log_t *log, u8 *key, u8 *hmac);
u8 add_log_entry(passman_log_t *log, ascii *data, u32 id);
u8 check_log(passman_log_t *log);
void print_log(passman_log_t *log);
