#include "error.h"
#include "types.h"
#include "memory.h"
#include "pbkdf2.h"

void passman_pbkdf2(void prf(u8 *, u64, u8 *, u64, u8 *), u64 prf_len,
		    u8 *in, u64 in_len, u8 *salt, u8 salt_len, u64 iter, u8 *key, u64 key_len)
{
  u32 key_blocks = key_len / prf_len;;
  
  u8 chash[PBKDF2_MAX_HASH_SIZE]; 
  u8 phash[PBKDF2_MAX_HASH_SIZE]; 
  u8 fhash[PBKDF2_MAX_HASH_SIZE]; 
  u8 asalt[PBKDF2_MAX_HASH_SIZE]; 
  
  for (u32 b = 1; b <= key_blocks; b++)
    {
      u32 be_b = rev32(b);
      
      passman_memcpy(asalt, salt, salt_len);
      passman_memcpy(asalt + salt_len, (u8 *)&be_b, 4);
      
      prf(asalt, salt_len + 4, in, in_len, phash);
      
      passman_memcpy(fhash, phash, prf_len);
      
      for (u64 i = 1; i < iter; i++)
	{
	  prf(phash, prf_len, in, in_len, chash);
	  
	  passman_memcpy(phash, chash, prf_len);
	  
	  for (u64 j = 0; j < prf_len; j++)
	    fhash[j] ^= chash[j]; 
	}
      
      passman_memcpy(key + ((b - 1) * prf_len), fhash, prf_len); 
    }
  
  passman_memzero(fhash, sizeof(fhash));
  passman_memzero(chash, sizeof(chash));
  passman_memzero(phash, sizeof(phash));
  passman_memzero(asalt, sizeof(asalt));
}
